"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Node = (function () {
    function Node(params) {
        this.position = params.position;
        this.costToChange = params.costToChange;
        this.path = undefined;
        this.distance = this.position.left + this.position.right;
    }
    Node.prototype.tryUpdate = function (path) {
        if (typeof this.path === 'undefined' || path.totalCost < this.path.totalCost) {
            this.path = path;
        }
    };
    return Node;
}());
var keyOf = function (leftIndex, rightIndex) { return leftIndex + "__" + rightIndex; };
var ArrayDiffElements = (function () {
    function ArrayDiffElements(params) {
        var _this = this;
        this.resolve = function () {
            var sortedNodes = _this.getSortedNodes();
            sortedNodes[0].path = { totalCost: 0, parent: undefined };
            sortedNodes.forEach(_this.updateTotalCostsNextTo);
            var lastNode = sortedNodes[sortedNodes.length - 1];
            var diffItems = _this.reproduceDiffFrom(lastNode);
            return _this.finalize(diffItems);
        };
        this.getSortedNodes = function () {
            return Array.from(_this.nodeByPosition.values()).sort(function (a, b) { return a.distance - b.distance; });
        };
        this.reproduceDiffFrom = function (node) {
            var go = function (n, diffItems) {
                var path = n.path;
                if (typeof path === 'undefined' || typeof path.parent === 'undefined') {
                    return diffItems;
                }
                else {
                    var parent_1 = path.parent;
                    switch (parent_1.operation) {
                        case 'Added':
                            var added = { type: 'Added', item: _this.right[parent_1.node.position.right] };
                            return go(parent_1.node, [added].concat(diffItems));
                        case 'Removed':
                            var removed = { type: 'Removed', item: _this.left[parent_1.node.position.left] };
                            return go(parent_1.node, [removed].concat(diffItems));
                        case 'Changed':
                            var changed = { type: 'Changed', left: _this.left[parent_1.node.position.left], right: _this.right[parent_1.node.position.right] };
                            return go(parent_1.node, [changed].concat(diffItems));
                        default:
                            var unchanged = { type: 'Unchanged', item: _this.right[parent_1.node.position.right] };
                            return go(parent_1.node, [unchanged].concat(diffItems));
                    }
                }
            };
            return go(node, []);
        };
        this.finalize = function (diffItems) {
            var merge = function (acc) { return acc.acc.concat(acc.currentRemoved, acc.currentAdded); };
            return merge(diffItems.reduce(function (_a, diffItem) {
                var acc = _a.acc, currentAdded = _a.currentAdded, currentRemoved = _a.currentRemoved;
                switch (diffItem.type) {
                    case 'Changed':
                    case 'Unchanged':
                        return { acc: merge({ acc: acc, currentAdded: currentAdded, currentRemoved: currentRemoved }).concat([diffItem]), currentAdded: [], currentRemoved: [] };
                    case 'Added':
                        return { acc: acc, currentRemoved: currentRemoved, currentAdded: currentAdded.concat([diffItem]) };
                    case 'Removed':
                        return { acc: acc, currentAdded: currentAdded, currentRemoved: currentRemoved.concat([diffItem]) };
                }
            }, { acc: [], currentAdded: [], currentRemoved: [] }));
        };
        this.updateTotalCostsNextTo = function (node) {
            var _a = node.position, left = _a.left, right = _a.right;
            var removed = _this.nodeOf(left + 1, right);
            if (typeof removed !== 'undefined') {
                removed.tryUpdate({
                    totalCost: node.path.totalCost + _this.costToRemove,
                    parent: { node: node, operation: 'Removed' },
                });
            }
            var added = _this.nodeOf(left, right + 1);
            if (typeof added !== 'undefined') {
                added.tryUpdate({
                    totalCost: node.path.totalCost + _this.costToAdd,
                    parent: { node: node, operation: 'Added' },
                });
            }
            var change = _this.nodeOf(left + 1, right + 1);
            if (typeof change !== 'undefined' && typeof change.costToChange !== 'undefined') {
                change.tryUpdate({
                    totalCost: node.path.totalCost + change.costToChange,
                    parent: { node: node, operation: change.costToChange === 0 ? 'Unchanged' : 'Changed' }
                });
            }
        };
        this.nodeOf = function (left, right) {
            return _this.nodeByPosition.get(keyOf(left, right));
        };
        var compare = params.compare;
        this.left = params.left;
        this.right = params.right;
        this.costToAdd = params.costToAdd;
        this.costToRemove = params.costToRemove;
        var nodesByPosition = new Map();
        for (var leftIndex = 0; leftIndex <= this.left.length; leftIndex++) {
            for (var rightIndex = 0; rightIndex <= this.right.length; rightIndex++) {
                var leftItem = this.left[leftIndex - 1];
                var rightItem = this.right[rightIndex - 1];
                var costToChange = (typeof leftItem !== 'undefined' && typeof rightItem !== 'undefined')
                    ? compare(leftItem, rightItem)
                    : undefined;
                nodesByPosition.set(keyOf(leftIndex, rightIndex), new Node({
                    position: { left: leftIndex, right: rightIndex },
                    costToChange: costToChange,
                }));
            }
        }
        this.nodeByPosition = nodesByPosition;
    }
    return ArrayDiffElements;
}());
exports.arrayDiff = function (left, right) { return function (compare) {
    return new ArrayDiffElements({ left: left, right: right, compare: compare, costToAdd: 0.5, costToRemove: 0.5 }).resolve();
}; };
//# sourceMappingURL=array-diff.js.map