"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var index_1 = __importDefault(require("./index"));
var testArrayDiff = function (conditions) { return function () {
    var left = conditions.left, right = conditions.right, compare = conditions.compare, expectedDiffTypes = conditions.expectedDiffTypes;
    var diffItems = index_1.default(left, right)(compare);
    expect(applyDiff(left, diffItems)).toEqual(right);
    expect(diffItems.map(function (item) { return item.type; })).toEqual(expectedDiffTypes);
}; };
var applyDiff = function (left, diffItems) {
    return diffItems.reduce(function (_a, diffItem) {
        var leftRemaining = _a.leftRemaining, acc = _a.acc;
        switch (diffItem.type) {
            case 'Added': return { leftRemaining: leftRemaining, acc: acc.concat([diffItem.item]) };
            case 'Removed': {
                var tail = leftRemaining.slice(1);
                return { leftRemaining: tail, acc: acc };
            }
            case 'Unchanged': {
                var tail = leftRemaining.slice(1);
                return { leftRemaining: tail, acc: acc.concat([diffItem.item]) };
            }
            case 'Changed': {
                var tail = leftRemaining.slice(1);
                return { leftRemaining: tail, acc: acc.concat([diffItem.right]) };
            }
        }
    }, { leftRemaining: left, acc: [] }).acc;
};
test('simple case', testArrayDiff({
    left: [1, 2, 3, 4, 22, 6],
    right: [2, 3, 4, 10, 6, 7],
    compare: function (a, b) { return a === b ? 0 : 0.9; },
    expectedDiffTypes: [
        'Removed',
        'Unchanged',
        'Unchanged',
        'Unchanged',
        'Changed',
        'Unchanged',
        'Added',
    ]
}));
test('ordered Remove/Add rather than change', testArrayDiff({
    left: [10, 11, 12, 13, 14],
    right: [20, 21, 22, 23, 24],
    compare: function (a, b) { return a === b ? 0 : 1.1; },
    expectedDiffTypes: [
        'Removed',
        'Removed',
        'Removed',
        'Removed',
        'Removed',
        'Added',
        'Added',
        'Added',
        'Added',
        'Added',
    ]
}));
test('both lists are empty', testArrayDiff({
    left: [],
    right: [],
    compare: function (a, b) { return a == b ? 0 : 2; },
    expectedDiffTypes: [],
}));
var makeRandomArray = function (num) { return Array.apply(null, Array(num)).map(function () { return Math.random(); }); };
var longArray1 = makeRandomArray(300);
var longArray2 = makeRandomArray(300);
test('rough performance check', function (done) {
    var start = new Date();
    index_1.default(longArray1, longArray2)(function (a, b) { return Math.abs(a - b) * 2; });
    setTimeout(function () {
        var end = new Date();
        expect(end.getTime() - start.getTime()).toBeLessThanOrEqual(1000);
        done();
    }, 0);
});
//# sourceMappingURL=index.test.js.map